# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-09-11 03:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("appmail", "0004_emailtemplate_is_active")]

    operations = [
        migrations.AddField(
            model_name="emailtemplate",
            name="from_email",
            field=models.CharField(
                default=settings.DEFAULT_FROM_EMAIL,
                help_text=(
                    "Default sender address if none specified. Verbose form is "
                    "accepted."
                ),
                max_length=254,
                verbose_name="Sender",
            ),
        ),
        migrations.AddField(
            model_name="emailtemplate",
            name="reply_to",
            field=models.CharField(
                default=settings.DEFAULT_FROM_EMAIL,
                help_text="Comma separated list of Reply-To recipients.",
                max_length=254,
                verbose_name="Reply-To",
            ),
        ),
        migrations.AlterField(
            model_name="emailtemplate",
            name="description",
            field=models.CharField(
                blank=True,
                help_text=(
                    "Optional description. e.g. used to differentiate variants ('new "
                    "header')."
                ),
                max_length=100,
                verbose_name="Description",
            ),
        ),
    ]
